@@ -13,30 +13,14 @@ class Authenticator:
     def __init__(self, ctx: "MemfaultCliClickContext"):
         self.ctx = ctx
 
-    @classmethod
-    def create_authenticator_given_context_or_raise(
-        cls, ctx: "MemfaultCliClickContext"
-    ) -> "Authenticator":
-        """
-        Given a CLI context, determine which auth system to use.
-        Simple implementation for now. If this becomes more complicated, iterate through
-        subclasses instead.
-        """
-        if ctx.obj.get("project_key"):
-            return ProjectKeyAuthenticator(ctx)
-        if ctx.obj.get("org_token"):
-            return OrgTokenAuthenticator(ctx)
-        else:
-            return BasicAuthenticator(ctx)
-
     @staticmethod
     @abstractmethod
     def project_key_auth() -> bool:
         pass
 
-    @staticmethod
+    @classmethod
     @abstractmethod
-    def required_args() -> List[str]:
+    def required_args(cls) -> List[str]:
         pass
 
     @abstractmethod
@@ -53,8 +37,8 @@ class ProjectKeyAuthenticator(Authenticator):
     def project_key_auth() -> bool:
         return True
 
-    @staticmethod
-    def required_args() -> List[str]:
+    @classmethod
+    def required_args(cls) -> List[str]:
         return [
             "project_key",
         ]
@@ -84,8 +68,8 @@ class BasicAuthenticator(Authenticator):
     def project_key_auth() -> bool:
         return False
 
-    @staticmethod
-    def required_args() -> List[str]:
+    @classmethod
+    def required_args(cls) -> List[str]:
         return [
             "org",
             "project",
@@ -107,8 +91,8 @@ class OrgTokenAuthenticator(Authenticator):
     def project_key_auth() -> bool:
         return False
 
-    @staticmethod
-    def required_args() -> List[str]:
+    @classmethod
+    def required_args(cls) -> List[str]:
         return [
             "org",
             "org_token",
