@@ -1,8 +1,12 @@
+import math
+import os
 import re
 from base64 import b64decode
 from typing import List
 
 import requests
+import tqdm
+from more_itertools import chunked
 from requests import PreparedRequest
 
 from .authenticator import Authenticator
@@ -26,10 +30,37 @@ class MemfaultChunk:
         response = requests.post(self.url, data=chunk, **request_args)
         if response.status_code >= 400:
             raise Exception(
-                f"Request failed with HTTP status {response.status_code}\nResponse body:\n{response.content.decode()}"
+                f"Request failed with HTTP status {response.status_code}\nResponse"
+                f" body:\n{response.content.decode()}"
+            )
+
+    def do_batch_post(self, files: List):
+        """Issue a post with a batch of files (HTTP Multipart)"""
+        request_args = self.authenticator.requests_auth_params()
+
+        # Pick up defaults (i.e User-Agent) and merge with Memfault Auth headers
+        headers = requests.utils.default_headers()
+        headers.update(request_args["headers"])
+        del request_args["headers"]
+
+        request = PreparedRequest()
+        request.prepare(method="POST", url=self.url, files=files, headers=headers, **request_args)
+        content_type: str = request.headers["content-type"]
+        request.headers["content-type"] = content_type.replace("form-data", "mixed")
+
+        session = requests.Session()
+        response = session.send(request)
+        if response.status_code >= 400:
+            raise Exception(
+                f"Request failed with HTTP status {response.status_code}\nResponse"
+                f" body:\n{response.content.decode()}"
             )
 
     def batch_post(self, chunks: List[bytes]):
+        """
+        Post a batch of chunks to the Memfault Chunk API. Will break the request
+        into multiple Multipart requests if the chunk count is too large
+        """
         if len(chunks) == 1:
             self.post(chunks[0])
             return
@@ -50,24 +81,22 @@ class MemfaultChunk:
                 )
             )
 
-        request_args = self.authenticator.requests_auth_params()
+        # by default, limit each request to 500 chunks. this is an extreme edge
+        # case but it can happen.
+        batch_size = int(os.environ.get("MEMFAULT_CLI_CHUNK_BATCH_SIZE", "500"))
 
-        # Pick up defaults (i.e User-Agent) and merge with Memfault Auth headers
-        headers = requests.utils.default_headers()
-        headers.update(request_args["headers"])
-        del request_args["headers"]
-
-        request = PreparedRequest()
-        request.prepare(method="POST", url=self.url, files=files, headers=headers, **request_args)
-        content_type: str = request.headers["content-type"]
-        request.headers["content-type"] = content_type.replace("form-data", "mixed")
-
-        session = requests.Session()
-        response = session.send(request)
-        if response.status_code >= 400:
-            raise Exception(
-                f"Request failed with HTTP status {response.status_code}\nResponse body:\n{response.content.decode()}"
-            )
+        if len(files) <= batch_size:
+            # do a single post if the chunk count is small enough
+            self.do_batch_post(files)
+        else:
+            # otherwise, break the request into multiple posts
+            for chunk_group in tqdm.tqdm(
+                chunked(files, batch_size),
+                total=math.ceil(len(files) / batch_size),
+                unit="batch",
+                desc=f"Uploading chunks in batches of {batch_size}",
+            ):
+                self.do_batch_post(chunk_group)
 
     @staticmethod
     def extract_exported_chunks(data: str):
