@@ -3,6 +3,7 @@ import logging
 import os
 import pathlib
 from base64 import b64decode
+from datetime import datetime, timezone
 from typing import List, Optional, Type
 
 import click
@@ -16,6 +17,7 @@ from .upload import (
     AndroidAppSymbolsUploader,
     BugreportUploader,
     CoredumpUploader,
+    CustomDataRecordingUploader,
     ElfSymbolDirectoryUploader,
     ElfSymbolUploader,
     MarUploader,
@@ -241,18 +243,33 @@ def upload_mar(ctx: MemfaultCliClickContext, path: str, **kwargs):
     "-i",
     required=True,
     help="""
-    The path to the image .tar/.tar.bz2/.tar.gz/.tar.xz as produced by Yocto's
-    do_image_*. The file is expected to reside in the location where Yocto
-    produced the file (tmp/deploy/images/${PACKAGE_ARCH}/images). Additionally,
-    an image with the same name but ending in -dbg.tar/.tar.bz2/.tar.gz is
-    expected to reside next to the main image. To generate it alongside the
-    required elfutils-native, add the following to your main image (note that
-    the main image will not be affected except for its build dependencies):
+    The path to the root filesystem image as produced by Yocto's do_image_*.
+    The file is expected to reside in the location where Yocto produced the
+    file (tmp/deploy/images/${PACKAGE_ARCH}/images).
+
+    Supported formats: .tar, .tar.bz2, .tar.gz, .tar.xz
+    """,
+    type=click.Path(exists=True, resolve_path=True, dir_okay=False, path_type=pathlib.Path),
+)
+@click.option(
+    "--dbg-image",
+    "-d",
+    required=True,
+    help="""
+    The path to the dbg filesystem image as produced by Yocto's
+    IMAGE_GEN_DEBUGFS option. The file is expected to reside in the location
+    where Yocto produced the file (tmp/deploy/images/${PACKAGE_ARCH}/images).
+
+    To generate it alongside the required elfutils-native, add the following to
+    your main image (note that the main image will not be affected except for
+    its build dependencies):
 
     \b
     DEPENDS:append = " elfutils-native"
     IMAGE_GEN_DEBUGFS = "2"
     IMAGE_FSTYPES_DEBUGFS = "tar.bz2"
+
+    Supported formats: .tar, .tar.bz2, .tar.gz, .tar.xz
     """,
     type=click.Path(exists=True, resolve_path=True, dir_okay=False, path_type=pathlib.Path),
 )
@@ -287,6 +304,7 @@ def upload_mar(ctx: MemfaultCliClickContext, path: str, **kwargs):
 def upload_yocto_symbols(
     ctx: MemfaultCliClickContext,
     image: pathlib.Path,
+    dbg_image: pathlib.Path,
     eu_unstrip_path: Optional[pathlib.Path],
     package_debug_split_style: Optional[str],
     **kwargs,
@@ -294,7 +312,7 @@ def upload_yocto_symbols(
     """Upload symbols for a Linux Yocto build.
 
     To see a full example, take a look at the Linux SDK example project:
-    https://mflt.io/yocto-symbols-image
+    https://mflt.io/yocto-upload-symbols
 
     Example Yocto Symbol Upload:
 
@@ -302,7 +320,9 @@ def upload_yocto_symbols(
         $ bitbake my-image
         $ memfault --org-token $ORG_TOKEN \\
                    --org acme-inc --project smart-sink \\
-            upload-yocto-symbols --image build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3.tar.bz2
+            upload-yocto-symbols \\
+                   --image build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3.tar.bz2 \\
+                   --dbg-image build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3-dbg.tar.bz2
     """
     ctx.obj.update(**kwargs)
 
@@ -318,7 +338,7 @@ def upload_yocto_symbols(
     else:
         build_env = get_necessary_from_bitbake_build_env()
 
-    process_and_upload_yocto_symbols(ctx, image, build_env=build_env)
+    process_and_upload_yocto_symbols(ctx, image, dbg_image, build_env=build_env)
 
 
 @cli.command(name="upload-mcu-symbols")
@@ -524,6 +544,64 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
     _do_upload_or_raise(ctx, path, ReleaseArtifactUploader)
 
 
+@cli.command(name="upload-custom-data-recording")
+@click.option(
+    "--hardware-version",
+    type=click.STRING,
+    required=True,
+    help="Required to identify the type of hardware.",
+)
+@click.option(
+    "--software-type",
+    type=click.STRING,
+    required=True,
+    help="Required to identify the system software",
+)
+@click.option(
+    "--software-version",
+    type=click.STRING,
+    required=True,
+    help="Required to identify single builds on Devices",
+)
+@click.option(
+    "--device-serial",
+    required=True,
+    type=click.STRING,
+    help="The unique identifier of a Device",
+)
+@click.option(
+    "--start-time",
+    required=False,
+    default=datetime.now(tz=timezone.utc).strftime("%Y-%m-%dT%H:%M:%S%z"),
+    type=click.DateTime(formats=("%Y-%m-%dT%H:%M:%S%z",)),
+    help="Timestamp of when the custom data recording started. The timezone needs to be specified. Defaults to the current time. Example: '2022-10-14T10:33:23Z' or '2022-10-14T10:33:23+0200'.",
+)
+@click.option(
+    "--duration-secs",
+    required=True,
+    type=click.INT,
+    help="The duration (in seconds) of the custom data recording",
+)
+@click.option(
+    "--mimetype",
+    required=True,
+    multiple=True,
+    help="(Can be given multiple times.) List of mimetypes to attach to the file.",
+)
+@click.option(
+    "--reason",
+    required=True,
+    type=click.STRING,
+    help="The reason this custom data recording was uploaded",
+)
+@click_argument_path
+@pass_memfault_cli_context
+def upload_custom_data_recording(ctx: MemfaultCliClickContext, path: str, **kwargs):
+    """Upload a custom data recording (read: any file that might help you with debugging)."""
+    ctx.obj.update(**kwargs)
+    _do_upload_or_raise(ctx, path, CustomDataRecordingUploader)
+
+
 @cli.command(name="deploy-release")
 @click.option("--release-version", type=str, required=False)
 @click.option("--delta-from", required=False)
