@@ -4,7 +4,7 @@ from typing import Dict, Tuple, Union
 import requests
 from requests import Session
 from requests.adapters import HTTPAdapter
-from urllib3 import Retry
+from requests.packages.urllib3.util.retry import Retry
 
 from .authenticator import Authenticator
 from .context import MemfaultCliClickContext
